# Generated by roxygen2: do not edit by hand

export(calculatePower)
export(diagnoseDataModel)
export(getEstimatesOfInterest)
export(readDesign)
export(readPowerParams)
export(readProbabilityModel)
exportClasses(PowerParams)
exportClasses(ProbabilityModel)
exportClasses(RMeDesign)
import(dplyr)
import(ggplot2)
import(magrittr)
import(methods)
importFrom(DHARMa,outliers)
importFrom(DHARMa,simulateResiduals)
importFrom(EnvStats,rosnerTest)
importFrom(ggtext,element_textbox_simple)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,mtext)
importFrom(influence.ME,cooks.distance.estex)
importFrom(influence.ME,grouping.levels)
importFrom(influence.ME,se.fixef)
importFrom(jsonlite,fromJSON)
importFrom(lme4,VarCorr)
importFrom(lme4,fixef)
importFrom(lme4,getME)
importFrom(lme4,glmer)
importFrom(lme4,glmer.nb)
importFrom(lme4,ranef)
importFrom(lmerTest,lmer)
importFrom(simr,"fixef<-")
importFrom(simr,extend)
importFrom(simr,fcompare)
importFrom(simr,fixed)
importFrom(simr,makeLmer)
importFrom(simr,powerCurve)
importFrom(simr,powerSim)
importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,residuals)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(tibble,rownames_to_column)
