% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readDesign.R
\name{readDesign}
\alias{readDesign}
\title{readDesign}
\usage{
readDesign(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary design parameters: condition_column, experimental_columns, response_column, total_column, condition_is_categorical, covariate, method, crossed_columns, error_is_non_normal, family_p, outlier_alpha, na.action}
}
\value{
an object of class RMeDesign
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
}
