% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{snRNAseq_gene_count_data}
\alias{snRNAseq_gene_count_data}
\title{Single-nucleus RNA-seq gene-level count data}
\format{
A data frame of gene-level counts and associated
  annotations.
}
\usage{
data(snRNAseq_gene_count_data)
}
\description{
Example dataset containing gene-level count summaries from a
single-nucleus RNA-seq experiment. This dataset can be used
to demonstrate power calculations for differential
expression-type analyses across experimental conditions.
}
\seealso{
\code{\link{snRNAseq_cluster_count_data}}
}
\keyword{datasets}
