% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readPowerParams.R
\name{readPowerParams}
\alias{readPowerParams}
\title{readPowerParams}
\usage{
readPowerParams(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary parameters for statistical power estimation: target_columns, power_curve, nsimn, levels, max_size, alpha, breaks, effect_size, icc}
}
\value{
an object of class PowerParams
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
}
