% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{mouse_brain_electro_physiology_data}
\alias{mouse_brain_electro_physiology_data}
\title{Mouse brain electrophysiology data}
\format{
A data frame of electrophysiological measurements and
  associated experimental annotations.
}
\usage{
data(mouse_brain_electro_physiology_data)
}
\description{
Example dataset containing electrophysiological measurements
from mouse brain recordings. The data are used to demonstrate
power analyses in experiments with repeated measurements and
complex correlation structures.
}
\keyword{datasets}
