% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readProbabilityModel.R
\name{readProbabilityModel}
\alias{readProbabilityModel}
\title{readProbabilityModel}
\usage{
readProbabilityModel(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary parameters for probability model: error_is_non_normal, family_p}
}
\value{
an object of class ProbabilityModel
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
}
