% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getEstimatesOfInterest.R
\name{getEstimatesOfInterest}
\alias{getEstimatesOfInterest}
\title{getEstimatesOfInterest}
\usage{
getEstimatesOfInterest(data, design, model, print_plots = TRUE)
}
\arguments{
\item{data}{Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest}

\item{design}{an object of class RMeDesign with the necessary design information about the data}

\item{model}{an object of class ProbabilityModel giving the error distribution of the data}

\item{print_plots}{Whether or not to print the plots, irrespective of this argument ggplot versions of evaluated association between the response_column and the condition_column. TRUE - print the plot, FALSE - do not print the plot}
}
\value{
a list with two elements - 1. an object of class summary.merMod and
2. the output from the get_residuals functions. This output consists of a list
with 3 elements. 1. The updated input data with an additional column with the model residuals of the individual observations. 2. A plot representing the purported association between the response column and the condition column. 3. The corresponding caption for this figure.
}
\description{
This function performs the estimations of interest and also visualizes the resulting association
}
\examples{
\donttest{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
data <- plate_assay_pilot_data
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
res <- getEstimatesOfInterest(data, design, model)
}
}
