% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_classes.R
\docType{class}
\name{ProbabilityModel-class}
\alias{ProbabilityModel-class}
\title{ProbabilityModel-class}
\arguments{
\item{error_is_non_normal}{Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).}

\item{family_p}{The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.}
}
\value{
an object of class ProbabilityModel
}
\description{
Objects of ProbabilityModel class store information on the assumed probability distribution for the model
}
\examples{
model=new("ProbabilityModel")
}
